@@ -162,9 +162,16 @@ public class SweetHome3DBootstrap {
         extensionJarsAndDlls.addAll(Arrays.asList(new String [] {
             "java3d-1.6/j3dcore.jar", // Java 3D 1.6 jars
             "java3d-1.6/vecmath.jar",
-            "java3d-1.6/j3dutils.jar",
+            "java3d-1.6/j3dutils.jar"}));
+        if ("64".equals(System.getProperty("sun.arch.data.model"))) {
+          extensionJarsAndDlls.addAll(Arrays.asList(new String [] {
             "java3d-1.6/gluegen-rt.jar",
             "java3d-1.6/jogl-java3d.jar"}));
+        } else {
+          extensionJarsAndDlls.addAll(Arrays.asList(new String [] {
+            "java3d-1.6/i586/gluegen-rt.jar",
+            "java3d-1.6/i586/jogl-java3d.jar"}));
+        }
         // Disable JOGL library loader
         System.setProperty("jogamp.gluegen.UseTempJarCache", "false");
         if ("64".equals(System.getProperty("sun.arch.data.model"))) {
