@@ -19,6 +19,7 @@
  */
 package com.eteks.sweethome3d.j3d;
 
+import com.eteks.sweethome3d.model.DimensionLine;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.Label;
@@ -35,6 +36,7 @@ import com.eteks.sweethome3d.viewcontroller.Object3DFactory;
  */
 public class Object3DBranchFactory implements Object3DFactory {
   private UserPreferences preferences;
+  private Object          context;
 
   public Object3DBranchFactory() {
     this(null);
@@ -44,24 +46,45 @@ public class Object3DBranchFactory implements Object3DFactory {
     this.preferences = preferences;
   }
 
+  public Object3DBranchFactory(UserPreferences preferences, Object context) {
+    this.preferences = preferences;
+    this.context = context;
+  }
+
   public boolean isDrawingModeEnabled() {
     return this.preferences != null && this.preferences.isDrawingModeEnabled();
   }
 
+  /**
+   * Returns the context of this factory used to {@linkplain #createObject3D(Home, Selectable, boolean) create 3D objects}.
+   */
+  public Object getContext() {
+    return this.context;
+  }
+
   /**
    * Returns the 3D object matching a given <code>item</code>.
    */
   public Object createObject3D(Home home, Selectable item, boolean waitForLoading) {
+    return createObject3D(home, item, this.preferences, getContext() != null  ? getContext()  : home, waitForLoading);
+  }
+
+  /**
+   * Returns the 3D object matching a given <code>item</code>.
+   */
+  public Object createObject3D(Home home, Selectable item, UserPreferences preferences, Object context, boolean waitForLoading) {
     if (item instanceof HomePieceOfFurniture) {
-      return new HomePieceOfFurniture3D((HomePieceOfFurniture)item, home, !isDrawingModeEnabled(), waitForLoading);
+      return new HomePieceOfFurniture3D((HomePieceOfFurniture)item, home, preferences, context, !isDrawingModeEnabled(), waitForLoading);
     } else if (item instanceof Wall) {
-      return new Wall3D((Wall)item, home, !isDrawingModeEnabled(), waitForLoading);
+      return new Wall3D((Wall)item, home, preferences, context, !isDrawingModeEnabled(), waitForLoading);
     } else if (item instanceof Room) {
-      return new Room3D((Room)item, home, false, !isDrawingModeEnabled(), waitForLoading);
+      return new Room3D((Room)item, home, preferences, context, false, !isDrawingModeEnabled(), waitForLoading);
     } else if (item instanceof Polyline) {
-      return new Polyline3D((Polyline)item, home);
+      return new Polyline3D((Polyline)item, home, preferences, context);
+    } else if (item instanceof DimensionLine) {
+      return new DimensionLine3D((DimensionLine)item, home, preferences, context, waitForLoading);
     } else if (item instanceof Label) {
-      return new Label3D((Label)item, home, waitForLoading);
+      return new Label3D((Label)item, home, preferences, context, waitForLoading);
     } else {
       return null;
     }
