@@ -419,7 +419,7 @@ public class CatalogLight extends CatalogPieceOfFurniture implements Light {
         lightSources, null, staircaseCutOutShape, modelRotation, backFaceShown ? SHOW_BACK_FACE : 0, modelSize, creator,
         resizable, deformable, texturable, horizontallyRotatable,
         price, valueAddedTaxPercentage, currency, properties);
-      }
+  }
 
   /**
    * Creates an unmodifiable catalog light of the default catalog.
@@ -470,11 +470,70 @@ public class CatalogLight extends CatalogPieceOfFurniture implements Light {
                       boolean resizable, boolean deformable, boolean texturable, boolean horizontallyRotatable,
                       BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency,
                       Map<String, String> properties) {
-    super(id, name, description, information, tags, creationDate, grade,
+    this(id, name, description, information, null, tags, creationDate, grade,
+        icon, planIcon, model, width, depth, height, elevation, dropOnTopElevation, movable,
+        lightSources, lightSourceMaterialNames, staircaseCutOutShape, modelRotation, modelFlags, modelSize, creator,
+        resizable, deformable, texturable, horizontallyRotatable,
+        price, valueAddedTaxPercentage, currency, properties, null);
+  }
+
+  /**
+   * Creates an unmodifiable catalog light of the default catalog.
+   * @param id    the id of the new light, or <code>null</code>
+   * @param name  the name of the new light
+   * @param description the description of the new light
+   * @param information additional information associated to the new light
+   * @param license license of the new light
+   * @param tags tags associated to the new light
+   * @param creationDate creation date of the new light in milliseconds since the epoch
+   * @param grade grade of the new light or <code>null</code>
+   * @param icon content of the icon of the new light
+   * @param planIcon content of the icon of the new piece displayed in plan
+   * @param model content of the 3D model of the new light
+   * @param width  the width in centimeters of the new light
+   * @param depth  the depth in centimeters of the new light
+   * @param height  the height in centimeters of the new light
+   * @param dropOnTopElevation a percentage of the height at which should be placed
+   *            an object dropped on the new piece
+   * @param elevation  the elevation in centimeters of the new light
+   * @param movable if <code>true</code>, the new light is movable
+   * @param lightSources the light sources of the new light
+   * @param lightSourceMaterialNames the material names of the light source in the 3D model of the new light
+   * @param staircaseCutOutShape the shape used to cut out upper levels when they intersect
+   *            with the piece like a staircase
+   * @param modelRotation the rotation 3 by 3 matrix applied to the light model
+   * @param modelFlags flags which should be applied to piece model
+   * @param modelSize size of the 3D model of the new light
+   * @param creator the creator of the model
+   * @param resizable if <code>true</code>, the size of the new light may be edited
+   * @param deformable if <code>true</code>, the width, depth and height of the new piece may
+   *            change independently from each other
+   * @param texturable if <code>false</code> this piece should always keep the same color or texture
+   * @param horizontallyRotatable if <code>false</code> this piece
+   *            should not rotate around an horizontal axis
+   * @param price the price of the new light, or <code>null</code>
+   * @param valueAddedTaxPercentage the Value Added Tax percentage applied to the
+   *             price of the new light or <code>null</code>
+   * @param currency the price currency, noted with ISO 4217 code, or <code>null</code>
+   * @param properties additional properties associating a key to a value or <code>null</code>
+   * @param contents   additional contents associating a key to a value or <code>null</code>
+   * @since 7.2
+   */
+  public CatalogLight(String id, String name, String description,
+                      String information, String license,
+                      String [] tags, Long creationDate, Float grade,
+                      Content icon, Content planIcon, Content model,
+                      float width, float depth, float height, float elevation, float dropOnTopElevation, boolean movable,
+                      LightSource [] lightSources, String [] lightSourceMaterialNames, String staircaseCutOutShape,
+                      float [][] modelRotation, int modelFlags, Long modelSize, String creator,
+                      boolean resizable, boolean deformable, boolean texturable, boolean horizontallyRotatable,
+                      BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency,
+                      Map<String, String> properties, Map<String, Content> contents) {
+    super(id, name, description, information, license, tags, creationDate, grade,
         icon, planIcon, model, width, depth, height, elevation, dropOnTopElevation, movable,
         staircaseCutOutShape, modelRotation, modelFlags, modelSize, creator,
         resizable, deformable, texturable, horizontallyRotatable,
-        price, valueAddedTaxPercentage, currency, properties);
+        price, valueAddedTaxPercentage, currency, properties, contents);
     this.lightSources = lightSources != null && lightSources.length > 0
         ? lightSources.clone()
         : EMPTY_LIGHT_SOURCES;
