@@ -28,13 +28,13 @@ import java.io.Serializable;
  */
 public class HomeMaterial implements Serializable {
   private static final long serialVersionUID = 1L;
-  
+
   private final String      name;
   private final String      key;
   private final Integer     color;
   private final HomeTexture texture;
   private final Float       shininess;
-  
+
   /**
    * Creates a material instance from parameters.
    * @since 4.0
@@ -63,10 +63,10 @@ public class HomeMaterial implements Serializable {
   public String getName() {
     return this.name;
   }
-  
+
   /**
    * Returns the key of this material. If not <code>null</code>, this key should be used
-   * as the unique identifier to find this material among the ones available on a model, 
+   * as the unique identifier to find this material among the ones available on a model,
    * rather than the name of this material.
    * @return the key of the material or <code>null</code> if material has no key.
    * @since 5.3
@@ -74,7 +74,7 @@ public class HomeMaterial implements Serializable {
   public String getKey() {
     return this.key;
   }
-  
+
   /**
    * Returns the color of this material.
    * @return the color of the material as RGB code or <code>null</code> if material color is unchanged.
@@ -83,7 +83,7 @@ public class HomeMaterial implements Serializable {
   public Integer getColor() {
     return this.color;
   }
-  
+
   /**
    * Returns the texture of this material.
    * @return the texture of the material or <code>null</code> if material texture is unchanged.
@@ -101,4 +101,51 @@ public class HomeMaterial implements Serializable {
   public Float getShininess() {
     return this.shininess;
   }
+
+  /**
+   * Returns <code>true</code> if this material is equal to <code>object</code>.
+   * @since 6.0
+   */
+  @Override
+  public boolean equals(Object object) {
+    if (object instanceof HomeMaterial) {
+      HomeMaterial material = (HomeMaterial)object;
+      return (material.name == this.name
+              || (material.name != null && material.name.equals(this.name)))
+          && (material.key == this.key
+              || (material.key != null && material.key.equals(this.name)))
+          && (material.color == this.color
+              || (material.color != null && material.color.equals(this.color)))
+          && (material.texture == this.texture
+              || (material.texture != null && material.texture.equals(this.texture)))
+          && (material.shininess == this.shininess
+              || (material.shininess != null && material.shininess.equals(this.shininess)));
+    }
+    return false;
+  }
+
+  /**
+   * Returns a hash code for this material.
+   * @since 6.0
+   */
+  @Override
+  public int hashCode() {
+    int hashCode = 0;
+    if (this.name != null) {
+      hashCode += this.name.hashCode();
+    }
+    if (this.key != null) {
+      hashCode += this.key.hashCode();
+    }
+    if (this.color != null) {
+      hashCode += this.color.hashCode();
+    }
+    if (this.texture != null) {
+      hashCode += this.texture.hashCode();
+    }
+    if (this.shininess != null) {
+      hashCode += this.shininess.hashCode();
+    }
+    return hashCode;
+  }
 }
