@@ -46,23 +46,25 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
   private static final double STRAIGHT_WALL_ANGLE_MARGIN  = Math.toRadians(1);
   private static final double ROUND_WALL_ANGLE_MARGIN     = Math.toRadians(10);
 
+  protected static final String [] EMPTY_PROPERTY_ARRAY = {};
+
   /**
    * The properties of a piece of furniture that may change. <code>PropertyChangeListener</code>s added
    * to a piece of furniture will be notified under a property name equal to the string value of one these properties.
    */
   public enum Property {CATALOG_ID, NAME, NAME_VISIBLE, NAME_X_OFFSET, NAME_Y_OFFSET, NAME_STYLE, NAME_ANGLE,
-      DESCRIPTION, INFORMATION, PRICE, VALUE_ADDED_TAX_PERCENTAGE, CURRENCY, ICON, PLAN_ICON, MODEL,
+      DESCRIPTION, INFORMATION, CREATOR, LICENSE, PRICE, VALUE_ADDED_TAX_PERCENTAGE, CURRENCY, ICON, PLAN_ICON, MODEL,
       WIDTH, WIDTH_IN_PLAN, DEPTH, DEPTH_IN_PLAN, HEIGHT, HEIGHT_IN_PLAN,
       COLOR, TEXTURE, MODEL_MATERIALS, MODEL_TRANSFORMATIONS,
-      STAIRCASE_CUT_OUT_SHAPE, CREATOR, SHININESS, VISIBLE,
+      STAIRCASE_CUT_OUT_SHAPE, SHININESS, VISIBLE,
       X, Y, ELEVATION, ANGLE, PITCH, ROLL, MODEL_ROTATION, MODEL_FLAGS, MODEL_MIRRORED,
       /** @deprecated */ BACK_FACE_SHOWN, MOVABLE, LEVEL};
 
   /**
    * The properties on which home furniture may be sorted.
    */
-  public enum SortableProperty {CATALOG_ID, NAME, WIDTH, DEPTH, HEIGHT, MOVABLE,
-                                DOOR_OR_WINDOW, COLOR, TEXTURE, VISIBLE, X, Y, ELEVATION, ANGLE, MODEL_SIZE, CREATOR,
+  public enum SortableProperty {CATALOG_ID, NAME, DESCRIPTION, CREATOR, LICENSE, WIDTH, DEPTH, HEIGHT, MOVABLE,
+                                DOOR_OR_WINDOW, COLOR, TEXTURE, VISIBLE, X, Y, ELEVATION, ANGLE, MODEL_SIZE,
                                 PRICE, VALUE_ADDED_TAX, VALUE_ADDED_TAX_PERCENTAGE, PRICE_VALUE_ADDED_TAX_INCLUDED, LEVEL};
   private static final Map<SortableProperty, Comparator<HomePieceOfFurniture>> SORTABLE_PROPERTY_COMPARATORS;
 
@@ -96,6 +98,45 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
           }
         }
       });
+    SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.DESCRIPTION, new Comparator<HomePieceOfFurniture>() {
+        public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
+          if (piece1.description == piece2.description) {
+            return 0;
+          } else if (piece1.description == null) {
+            return -1;
+          } else if (piece2.description == null) {
+            return 1;
+          } else {
+            return collator.compare(piece1.description, piece2.description);
+          }
+        }
+      });
+    SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.CREATOR, new Comparator<HomePieceOfFurniture>() {
+        public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
+          if (piece1.creator == piece2.creator) {
+            return 0;
+          } else if (piece1.creator == null) {
+            return -1;
+          } else if (piece2.creator == null) {
+            return 1;
+          } else {
+            return collator.compare(piece1.creator, piece2.creator);
+          }
+        }
+      });
+    SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.LICENSE, new Comparator<HomePieceOfFurniture>() {
+        public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
+          if (piece1.license == piece2.license) {
+            return 0;
+          } else if (piece1.license == null) {
+            return -1;
+          } else if (piece2.license == null) {
+            return 1;
+          } else {
+            return collator.compare(piece1.license, piece2.license);
+          }
+        }
+      });
     SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.WIDTH, new Comparator<HomePieceOfFurniture>() {
         public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
           return HomePieceOfFurniture.compare(piece1.width, piece2.width);
@@ -189,19 +230,6 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
           }
         }
       });
-    SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.CREATOR, new Comparator<HomePieceOfFurniture>() {
-        public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
-          if (piece1.creator == piece2.creator) {
-            return 0;
-          } else if (piece1.creator == null) {
-            return -1;
-          } else if (piece2.creator == null) {
-            return 1;
-          } else {
-            return collator.compare(piece1.creator, piece2.creator);
-          }
-        }
-      });
     SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.LEVEL, new Comparator<HomePieceOfFurniture>() {
         public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
           return HomePieceOfFurniture.compare(piece1.getLevel(), piece2.getLevel());
@@ -289,6 +317,8 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
   private float                  nameAngle;
   private String                 description;
   private String                 information;
+  private String                 creator;
+  private String                 license;
   private Content                icon;
   private Content                planIcon;
   private Content                model;
@@ -312,7 +342,6 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
   private boolean                modelCenteredAtOrigin;
   private Transformation []      modelTransformations;
   private String                 staircaseCutOutShape;
-  private String                 creator;
   private boolean                backFaceShown; // Used only for backward compatibility from version 7.0
   private boolean                resizable;
   private boolean                deformable;
@@ -335,23 +364,50 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
 
   /**
    * Creates a home piece of furniture from an existing piece.
+   * No additional properties will be copied.
    * @param piece the piece from which data are copied
    */
   public HomePieceOfFurniture(PieceOfFurniture piece) {
-    this(createId("pieceOfFurniture"), piece);
+    this(piece, EMPTY_PROPERTY_ARRAY);
+  }
+
+  /**
+   * Creates a home piece of furniture from an existing piece.
+   * @param piece the piece from which data are copied
+   * @param copiedProperties the names of the additional properties which should be copied from the existing piece
+   *                         or <code>null</code> if all properties should be copied.
+   * @since 7.2
+   */
+  public HomePieceOfFurniture(PieceOfFurniture piece, String [] copiedProperties) {
+    this(createId("pieceOfFurniture"), piece, copiedProperties);
+  }
+
+  /**
+   * Creates a home piece of furniture from an existing piece.
+   * No additional properties will be copied.
+   * @param id    the ID of the piece
+   * @param piece the piece from which data are copied
+   * @since 6.4
+   */
+  public HomePieceOfFurniture(String id, PieceOfFurniture piece) {
+    this(id, piece, EMPTY_PROPERTY_ARRAY);
   }
 
   /**
    * Creates a home piece of furniture from an existing piece.
    * @param id    the ID of the piece
    * @param piece the piece from which data are copied
-   * @since 6.4
+   * @param copiedProperties the names of the additional properties which should be copied from the existing piece
+   *                         or <code>null</code> if all properties should be copied.
+   * @since 7.2
    */
-  public HomePieceOfFurniture(String id, PieceOfFurniture piece) {
+  public HomePieceOfFurniture(String id, PieceOfFurniture piece, String [] copiedProperties) {
     super(id);
     this.name = piece.getName();
     this.description = piece.getDescription();
     this.information = piece.getInformation();
+    this.creator = piece.getCreator();
+    this.license = piece.getLicense();
     this.icon = piece.getIcon();
     this.planIcon = piece.getPlanIcon();
     this.model = piece.getModel();
@@ -366,7 +422,6 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     this.color = piece.getColor();
     this.modelRotation = piece.getModelRotation();
     this.staircaseCutOutShape = piece.getStaircaseCutOutShape();
-    this.creator = piece.getCreator();
     this.modelFlags = piece.getModelFlags();
     this.resizable = piece.isResizable();
     this.deformable = piece.isDeformable();
@@ -398,9 +453,26 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
       this.texture = homePiece.getTexture();
       this.shininess = homePiece.getShininess();
       this.modelMaterials = homePiece.getModelMaterials();
+      for (String property : copiedProperties != null ? Arrays.asList(copiedProperties) : homePiece.getPropertyNames()) {
+        Object value = homePiece.isContentProperty(property)
+            ? homePiece.getContentProperty(property)
+            : homePiece.getProperty(property);
+        if (value != null) {
+          setProperty(property, value);
+        }
+      }
     } else {
       if (piece instanceof CatalogPieceOfFurniture) {
-        this.catalogId = ((CatalogPieceOfFurniture)piece).getId();
+        CatalogPieceOfFurniture catalogPiece = (CatalogPieceOfFurniture)piece;
+        this.catalogId = catalogPiece.getId();
+        for (String property : copiedProperties != null ? Arrays.asList(copiedProperties) : catalogPiece.getPropertyNames()) {
+          Object value = catalogPiece.isContentProperty(property)
+              ? catalogPiece.getContentProperty(property)
+              : catalogPiece.getProperty(property);
+          if (value != null) {
+            setProperty(property, value);
+          }
+        }
       }
       this.visible = true;
       this.widthInPlan = this.width;
@@ -632,6 +704,50 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     }
   }
 
+  /**
+   * Returns the creator of this piece.
+   * @since 4.2
+   */
+  public String getCreator() {
+    return this.creator;
+  }
+
+  /**
+   * Sets the creator of this piece. Once this piece is updated, listeners added to this piece
+   * will receive a change notification.
+   * @since 6.5
+   */
+  public void setCreator(String creator) {
+    if (creator != this.creator
+        && (creator == null || !creator.equals(this.creator))) {
+      String oldCreator = this.creator;
+      this.creator = creator;
+      firePropertyChange(Property.CREATOR.name(), oldCreator, creator);
+    }
+  }
+
+  /**
+   * Returns the license of this piece, or <code>null</code>.
+   * @since 7.2
+   */
+  public String getLicense() {
+    return this.license;
+  }
+
+  /**
+   * Sets the the license of this piece . Once this piece is updated,
+   * listeners added to this piece will receive a change notification.
+   * @since 7.2
+   */
+  public void setLicense(String license) {
+    if (license != this.license
+        && (license == null || !license.equals(this.license))) {
+      String oldLicense = this.license;
+      this.license = license;
+      firePropertyChange(Property.LICENSE.name(), oldLicense, license);
+    }
+  }
+
   /**
    * Returns the depth of this piece of furniture.
    */
@@ -1484,28 +1600,6 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     }
   }
 
-  /**
-   * Returns the creator of this piece.
-   * @since 4.2
-   */
-  public String getCreator() {
-    return this.creator;
-  }
-
-  /**
-   * Sets the creator of this piece. Once this piece is updated, listeners added to this piece
-   * will receive a change notification.
-   * @since 6.5
-   */
-  public void setCreator(String creator) {
-    if (creator != this.creator
-        && (creator == null || !creator.equals(this.creator))) {
-      String oldCreator = this.creator;
-      this.creator = creator;
-      firePropertyChange(Property.CREATOR.name(), oldCreator, creator);
-    }
-  }
-
   /**
    * Returns the level which this piece belongs to.
    * @since 3.4
@@ -1742,7 +1836,7 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
   }
 
   /**
-   * Returns a comparator that compares furniture on a given <code>property</code> in ascending order.
+   * Returns a comparator which compares furniture on a given <code>property</code> in ascending order.
    */
   public static Comparator<HomePieceOfFurniture> getFurnitureComparator(SortableProperty property) {
     return SORTABLE_PROPERTY_COMPARATORS.get(property);
