@@ -55,7 +55,7 @@ public abstract class UserPreferences {
                         FURNITURE_VIEWED_FROM_TOP, FURNITURE_MODEL_ICON_SIZE, ROOM_FLOOR_COLORED_OR_TEXTURED, WALL_PATTERN, NEW_WALL_PATTERN,
                         NEW_WALL_THICKNESS, NEW_WALL_HEIGHT, NEW_WALL_SIDEBOARD_THICKNESS, NEW_WALL_SIDEBOARD_HEIGHT, NEW_ROOM_FLOOR_COLOR, NEW_FLOOR_THICKNESS,
                         RECENT_HOMES, IGNORED_ACTION_TIP, FURNITURE_CATALOG_VIEWED_IN_TREE, NAVIGATION_PANEL_VISIBLE,
-                        AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED, OBSERVER_CAMERA_SELECTED_AT_CHANGE, CHECK_UPDATES_ENABLED,
+                        AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED, OBSERVER_CAMERA_SELECTED_AT_CHANGE, EDITING_IN_3D_VIEW_ENABLED, CHECK_UPDATES_ENABLED,
                         UPDATES_MINIMUM_DATE, AUTO_SAVE_DELAY_FOR_RECOVERY, AUTO_COMPLETION_STRINGS, RECENT_COLORS, RECENT_TEXTURES, HOME_EXAMPLES, PHOTO_RENDERER}
 
   public static final String FURNITURE_LIBRARY_TYPE = "Furniture library";
@@ -102,9 +102,10 @@ public abstract class UserPreferences {
   private BigDecimal       defaultValueAddedTaxPercentage;
   private LengthUnit       unit;
   private boolean          furnitureCatalogViewedInTree = true;
+  private boolean          navigationPanelVisible = true;
+  private boolean          editingIn3DViewEnabled;
   private boolean          aerialViewCenteredOnSelectionEnabled;
   private boolean          observerCameraSelectedAtChange = true;
-  private boolean          navigationPanelVisible = true;
   private boolean          magnetismEnabled    = true;
   private boolean          rulersVisible       = true;
   private boolean          gridVisible         = true;
@@ -642,14 +643,22 @@ public abstract class UserPreferences {
   }
 
   /**
-   * Sets whether aerial view should be centered on selection or not.
-   * @since 4.0
+   * Returns whether interactive editing in 3D view is enabled or not.
+   * @since 7.2
    */
-  public void setAerialViewCenteredOnSelectionEnabled(boolean aerialViewCenteredOnSelectionEnabled) {
-    if (aerialViewCenteredOnSelectionEnabled != this.aerialViewCenteredOnSelectionEnabled) {
-      this.aerialViewCenteredOnSelectionEnabled = aerialViewCenteredOnSelectionEnabled;
-      this.propertyChangeSupport.firePropertyChange(Property.AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED.name(),
-          !aerialViewCenteredOnSelectionEnabled, aerialViewCenteredOnSelectionEnabled);
+  public boolean isEditingIn3DViewEnabled() {
+    return this.editingIn3DViewEnabled;
+  }
+
+  /**
+   * Sets whether interactive editing in 3D view is enabled or not.
+   * @since 7.2
+   */
+  public void setEditingIn3DViewEnabled(boolean editingIn3DViewEnabled) {
+    if (editingIn3DViewEnabled != this.editingIn3DViewEnabled) {
+      this.editingIn3DViewEnabled = editingIn3DViewEnabled;
+      this.propertyChangeSupport.firePropertyChange(Property.EDITING_IN_3D_VIEW_ENABLED.name(),
+          !editingIn3DViewEnabled, editingIn3DViewEnabled);
     }
   }
 
@@ -662,14 +671,14 @@ public abstract class UserPreferences {
   }
 
   /**
-   * Sets whether the observer camera should be selected at each change.
-   * @since 5.5
+   * Sets whether aerial view should be centered on selection or not.
+   * @since 4.0
    */
-  public void setObserverCameraSelectedAtChange(boolean observerCameraSelectedAtChange) {
-    if (observerCameraSelectedAtChange != this.observerCameraSelectedAtChange) {
-      this.observerCameraSelectedAtChange = observerCameraSelectedAtChange;
-      this.propertyChangeSupport.firePropertyChange(Property.OBSERVER_CAMERA_SELECTED_AT_CHANGE.name(),
-          !observerCameraSelectedAtChange, observerCameraSelectedAtChange);
+  public void setAerialViewCenteredOnSelectionEnabled(boolean aerialViewCenteredOnSelectionEnabled) {
+    if (aerialViewCenteredOnSelectionEnabled != this.aerialViewCenteredOnSelectionEnabled) {
+      this.aerialViewCenteredOnSelectionEnabled = aerialViewCenteredOnSelectionEnabled;
+      this.propertyChangeSupport.firePropertyChange(Property.AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED.name(),
+          !aerialViewCenteredOnSelectionEnabled, aerialViewCenteredOnSelectionEnabled);
     }
   }
 
@@ -681,6 +690,18 @@ public abstract class UserPreferences {
     return this.observerCameraSelectedAtChange;
   }
 
+  /**
+   * Sets whether the observer camera should be selected at each change.
+   * @since 5.5
+   */
+  public void setObserverCameraSelectedAtChange(boolean observerCameraSelectedAtChange) {
+    if (observerCameraSelectedAtChange != this.observerCameraSelectedAtChange) {
+      this.observerCameraSelectedAtChange = observerCameraSelectedAtChange;
+      this.propertyChangeSupport.firePropertyChange(Property.OBSERVER_CAMERA_SELECTED_AT_CHANGE.name(),
+          !observerCameraSelectedAtChange, observerCameraSelectedAtChange);
+    }
+  }
+
   /**
    * Returns <code>true</code> if magnetism is enabled.
    * @return <code>true</code> by default.
