@@ -20,7 +20,6 @@
 package com.eteks.sweethome3d.swing;
 
 import java.awt.Component;
-import java.awt.ComponentOrientation;
 import java.awt.event.ContainerEvent;
 import java.awt.event.ContainerListener;
 import java.beans.PropertyChangeEvent;
@@ -42,9 +41,9 @@ public class UnfocusableToolBar extends JToolBar {
    * Creates an unfocusable toolbar.
    */
   public UnfocusableToolBar() {
-    // Update toolBar buttons when component orientation changes 
-    // and when buttons are added or removed to it  
-    addPropertyChangeListener("componentOrientation", 
+    // Update toolBar buttons when component orientation changes
+    // and when buttons are added or removed to it
+    addPropertyChangeListener("componentOrientation",
         new PropertyChangeListener () {
           public void propertyChange(PropertyChangeEvent evt) {
             updateToolBarButtons();
@@ -54,29 +53,28 @@ public class UnfocusableToolBar extends JToolBar {
         public void componentAdded(ContainerEvent ev) {
           updateToolBarButtons();
         }
-        
+
         public void componentRemoved(ContainerEvent ev) {}
       });
   }
 
   /**
-   * Ensures that all the children of this tool bar aren't focusable. 
+   * Ensures that all the children of this tool bar aren't focusable.
    * Under Mac OS X 10.5, it also uses segmented buttons and groups them depending
    * on toolbar orientation and whether a button is after or before a separator.
    */
   private void updateToolBarButtons() {
-    // Retrieve component orientation because Mac OS X 10.5 doesn't take it into account 
-    ComponentOrientation orientation = getComponentOrientation();
+    // Retrieve component orientation because Mac OS X 10.5 doesn't take it into account
     Component previousComponent = null;
-    for (int i = 0, n = getComponentCount(); i < n; i++) {        
-      JComponent component = (JComponent)getComponentAtIndex(i); 
+    for (int i = 0, n = getComponentCount(); i < n; i++) {
+      JComponent component = (JComponent)getComponentAtIndex(i);
       // Remove focusable property on buttons
       component.setFocusable(false);
-      
+
       if (!(component instanceof AbstractButton)) {
         previousComponent = null;
         continue;
-      }          
+      }
       if (OperatingSystem.isMacOSXLeopardOrSuperior()) {
         Component nextComponent;
         if (i < n - 1) {
@@ -89,15 +87,9 @@ public class UnfocusableToolBar extends JToolBar {
             && !(nextComponent instanceof AbstractButton)) {
           component.putClientProperty("JButton.segmentPosition", "only");
         } else if (previousComponent == null) {
-          component.putClientProperty("JButton.segmentPosition", 
-              orientation.isLeftToRight() 
-                ? "first"
-                : "last");
+          component.putClientProperty("JButton.segmentPosition", "first");
         } else if (!(nextComponent instanceof AbstractButton)) {
-          component.putClientProperty("JButton.segmentPosition",
-              orientation.isLeftToRight() 
-                ? "last"
-                : "first");
+          component.putClientProperty("JButton.segmentPosition", "last");
         } else {
           component.putClientProperty("JButton.segmentPosition", "middle");
         }
