@@ -26,13 +26,19 @@ import java.math.BigDecimal;
 import java.math.RoundingMode;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
 import java.util.Currency;
+import java.util.HashMap;
+import java.util.LinkedHashMap;
 import java.util.List;
+import java.util.Map;
 
 import javax.swing.undo.CannotRedoException;
 import javax.swing.undo.CannotUndoException;
 import javax.swing.undo.UndoableEditSupport;
 
+import com.eteks.sweethome3d.model.CatalogPieceOfFurniture;
 import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.HomeDoorOrWindow;
@@ -41,6 +47,7 @@ import com.eteks.sweethome3d.model.HomeLight;
 import com.eteks.sweethome3d.model.HomeMaterial;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.HomeTexture;
+import com.eteks.sweethome3d.model.ObjectProperty;
 import com.eteks.sweethome3d.model.Sash;
 import com.eteks.sweethome3d.model.Selectable;
 import com.eteks.sweethome3d.model.Transformation;
@@ -54,7 +61,7 @@ public class HomeFurnitureController implements Controller {
   /**
    * The properties that may be edited by the view associated to this controller.
    */
-  public enum Property {ICON, NAME, NAME_VISIBLE, DESCRIPTION, PRICE, VALUE_ADDED_TAX_PERCENTAGE,
+  public enum Property {ICON, NAME, NAME_VISIBLE, DESCRIPTION, ADDITIONAL_PROPERTIES, PRICE, VALUE_ADDED_TAX_PERCENTAGE,
       X, Y, ELEVATION, BASE_PLAN_ITEM,
       ANGLE, ANGLE_IN_DEGREES, ROLL, PITCH, HORIZONTAL_AXIS, WIDTH, DEPTH, HEIGHT, PROPORTIONAL,
       COLOR, PAINT, SHININESS, VISIBLE, MODEL_MIRRORED, MODEL_TRANSFORMATIONS, LIGHT_POWER,
@@ -89,6 +96,7 @@ public class HomeFurnitureController implements Controller {
   private Content            icon;
   private String             name;
   private String             description;
+  private Map<ObjectProperty, Object> additionalProperties;
   private boolean            priceEditable;
   private BigDecimal         price;
   private boolean            valueAddedTaxPercentageEditable;
@@ -111,6 +119,8 @@ public class HomeFurnitureController implements Controller {
   private Float              proportionalHeight;
   private boolean            proportional;
   private Transformation []  modelTransformations;
+  private List<String>            modelPresetTransformationsNames;
+  private List<Transformation []> modelPresetTransformations;
   private Integer            color;
   private FurniturePaint     paint;
   private FurnitureShininess shininess;
@@ -150,7 +160,7 @@ public class HomeFurnitureController implements Controller {
    */
   public HomeFurnitureController(Home home,
                                  UserPreferences preferences,
-                                 ViewFactory viewFactory,
+                                 ViewFactory     viewFactory,
                                  ContentManager  contentManager,
                                  UndoableEditSupport undoSupport) {
     this.home = home;
@@ -233,6 +243,13 @@ public class HomeFurnitureController implements Controller {
     return this.homeFurnitureView;
   }
 
+  /**
+   * Returns the content manager associated to this controller.
+   */
+  public ContentManager getContentManager() {
+    return this.contentManager;
+  }
+
   /**
    * Displays the view controlled by this controller.
    */
@@ -267,6 +284,7 @@ public class HomeFurnitureController implements Controller {
       setName(null); // Nothing to edit
       setNameVisible(null);
       setDescription(null);
+      setAdditionalProperties(null);
       setPrice(null, false);
       this.priceEditable = false;
       setValueAddedTaxPercentage(null);
@@ -294,6 +312,8 @@ public class HomeFurnitureController implements Controller {
       }
       setPaint(null);
       setModelTransformations(null);
+      this.modelPresetTransformationsNames = Collections.emptyList();
+      this.modelPresetTransformations = Collections.emptyList();
       this.doorOrWindow = false;
       this.wallThickness = 1;
       this.wallDistance = 0;
@@ -357,6 +377,22 @@ public class HomeFurnitureController implements Controller {
       }
       setDescription(description);
 
+      if (selectedFurniture.size() == 1) {
+        Map<ObjectProperty, Object> additionalProperties = new LinkedHashMap<ObjectProperty, Object>();
+        for (ObjectProperty property : this.home.getFurnitureAdditionalProperties()) {
+          String propertyName = property.getName();
+          if (property.isModifiable()) {
+            additionalProperties.put(property,
+                firstPiece.isContentProperty(propertyName)
+                   ? firstPiece.getContentProperty(propertyName)
+                   : firstPiece.getProperty(propertyName));
+          }
+        }
+        setAdditionalProperties(additionalProperties.size() > 0 ? additionalProperties : null);
+      } else {
+        setAdditionalProperties(null);
+      }
+
       boolean priceEditable = this.preferences.getCurrency() != null;
       if (priceEditable) {
         for (int i = 0; i < selectedFurniture.size(); i++) {
@@ -600,7 +636,7 @@ public class HomeFurnitureController implements Controller {
       // Allow transformation change only on one piece at a time
       Transformation [] modelTransformations = firstPiece.getModelTransformations();
       if (selectedFurniture.size() != 1
-          || selectedFurniture.get(0) instanceof HomeFurnitureGroup) {
+          || firstPiece instanceof HomeFurnitureGroup) {
         modelTransformations = null;
       } else {
         if (modelTransformations == null) {
@@ -620,6 +656,48 @@ public class HomeFurnitureController implements Controller {
       }
       setModelTransformations(modelTransformations);
 
+      this.modelPresetTransformationsNames = new ArrayList<String>();
+      this.modelPresetTransformations = new ArrayList<Transformation[]>();
+      if (selectedFurniture.size() == 1
+          && !(firstPiece instanceof HomeFurnitureGroup)
+          && firstPiece.getCatalogId() != null) {
+        CatalogPieceOfFurniture catalogPiece = this.preferences.getFurnitureCatalog().getPieceOfFurnitureWithId(firstPiece.getCatalogId());
+        if (catalogPiece != null) {
+          int i = 1;
+          // Parse preset transformations from additional properties modelPresetTransformationsName_x and modelPresetTransformations_x stored in catalog only
+          for (String presetTransformationsName = catalogPiece
+              .getProperty("modelPresetTransformationsName_"
+                           + i); presetTransformationsName != null; presetTransformationsName = catalogPiece
+                               .getProperty("modelPresetTransformationsName_" + ++i)) {
+            String presetTransformations = catalogPiece.getProperty("modelPresetTransformations_" + i);
+            if (presetTransformations != null) {
+              String [] strings = presetTransformations.trim().split("\\s+");
+              if (strings.length % 13 == 0) {
+                Transformation [] transformations = new Transformation [strings.length / 13];
+                try {
+                  for (int j = 0; j < strings.length; j += 12) {
+                    String transformationName = strings [j++];
+                    float [] [] matrix = new float [] [] {
+                      {Float.parseFloat(strings [j + 0]), Float.parseFloat(strings [j + 1]),
+                        Float.parseFloat(strings [j + 2]), Float.parseFloat(strings [j + 3])},
+                      {Float.parseFloat(strings [j + 4]), Float.parseFloat(strings [j + 5]),
+                        Float.parseFloat(strings [j + 6]), Float.parseFloat(strings [j + 7])},
+                      {Float.parseFloat(strings [j + 8]), Float.parseFloat(strings [j + 9]),
+                        Float.parseFloat(strings [j + 10]), Float.parseFloat(strings [j + 11])}};
+                    transformations [j / 13] = new Transformation(transformationName, matrix);
+                  }
+                  this.modelPresetTransformationsNames.add(presetTransformationsName);
+                  this.modelPresetTransformations.add(transformations);
+                } catch (NumberFormatException ex) {
+                  System.out.println(
+                      "Invalid value in preset transformations matrices for " + firstPiece.getCatalogId() + "\n" + ex);
+                }
+              }
+            }
+          }
+        }
+      }
+
       Float firstPieceShininess = firstPieceExceptGroup.getShininess();
       FurnitureShininess shininess = firstPieceShininess == null
           ? FurnitureShininess.DEFAULT
@@ -757,12 +835,11 @@ public class HomeFurnitureController implements Controller {
    * Returns <code>true</code> if the given <code>property</code> is editable.
    * Depending on whether a property is editable or not, the view associated to this controller
    * may render it differently.
-   * The implementation of this method always returns <code>true</code> except for <code>DESCRIPTION</code> and <code>PRICE</code> properties.
    */
   public boolean isPropertyEditable(Property property) {
     switch (property) {
-      case DESCRIPTION :
-        return false;
+      case ADDITIONAL_PROPERTIES :
+        return getAdditionalProperties() != null;
       case PRICE :
         return isPriceEditable();
       case VALUE_ADDED_TAX_PERCENTAGE :
@@ -855,6 +932,29 @@ public class HomeFurnitureController implements Controller {
     return this.description;
   }
 
+  /**
+   * Sets additional edited properties.
+   * @since 7.2
+   */
+  public void setAdditionalProperties(Map<ObjectProperty, Object> additionalProperties) {
+    if (additionalProperties != this.additionalProperties
+        && (additionalProperties == null || !additionalProperties.equals(this.additionalProperties))) {
+      Map<ObjectProperty, Object> oldAdditionalProperties = this.additionalProperties;
+      this.additionalProperties = additionalProperties != null
+          ? new LinkedHashMap<ObjectProperty, Object>(additionalProperties)
+          : null;
+      this.propertyChangeSupport.firePropertyChange(Property.ADDITIONAL_PROPERTIES.name(), oldAdditionalProperties, additionalProperties);
+    }
+  }
+
+  /**
+   * Returns additional edited properties.
+   * @since 7.2
+   */
+  public Map<ObjectProperty, Object> getAdditionalProperties() {
+    return this.additionalProperties;
+  }
+
   /**
    * Sets the edited price.
    * @since 4.0
@@ -1427,6 +1527,22 @@ public class HomeFurnitureController implements Controller {
     return this.modelTransformations;
   }
 
+  /**
+   * Returns the names of the available preset model transformations.
+   * @since 7.2
+   */
+  public List<String> getModelPresetTransformationsNames() {
+    return Collections.unmodifiableList(this.modelPresetTransformationsNames);
+  }
+
+  /**
+   * Returns the preset model transformations at the given <code>index</code>.
+   * @since 7.2
+   */
+  public Transformation [] getModelPresetTransformations(int index) {
+    return this.modelPresetTransformations.get(index);
+  }
+
   /**
    * Sets whether the piece shininess is the default one, matt, shiny or unknown.
    */
@@ -1570,6 +1686,16 @@ public class HomeFurnitureController implements Controller {
       String name = getName();
       Boolean nameVisible = getNameVisible();
       String description = getDescription();
+      Map<ObjectProperty, Object> editedProperties = getAdditionalProperties();
+      Map<String, Object> additionalProperties;
+      if (editedProperties == null) {
+        additionalProperties = null;
+      } else {
+        additionalProperties = new HashMap<String, Object>();
+        for (Map.Entry<ObjectProperty, Object> entry : editedProperties.entrySet()) {
+          additionalProperties.put(entry.getKey().getName(), entry.getValue());
+        }
+      }
       BigDecimal price = getPrice();
       boolean removePrice = selectedFurniture.size() == 1 && price == null;
       BigDecimal valueAddedTaxPercentage = getValueAddedTaxPercentage();
@@ -1632,7 +1758,7 @@ public class HomeFurnitureController implements Controller {
         }
       }
       // Apply modification
-      doModifyFurniture(modifiedFurniture, name, nameVisible, description,
+      doModifyFurniture(modifiedFurniture, name, nameVisible, description, additionalProperties,
           price, removePrice, valueAddedTaxPercentage, removeValueAddedTaxPercentage, currency,
           x, y, elevation, angle, roll, pitch, horizontalAxis, basePlanItem,
           width, depth, height, proportional, modelTransformations,
@@ -1642,7 +1768,7 @@ public class HomeFurnitureController implements Controller {
         List<Selectable> newSelection = this.home.getSelectedItems();
         this.undoSupport.postEdit(new FurnitureModificationUndoableEdit(this.home, this.preferences,
             oldSelection.toArray(new Selectable [oldSelection.size()]), newSelection.toArray(new Selectable [newSelection.size()]),
-            modifiedFurniture, name, nameVisible, description, price, removePrice, valueAddedTaxPercentage, removeValueAddedTaxPercentage, currency,
+            modifiedFurniture, name, nameVisible, description, additionalProperties, price, removePrice, valueAddedTaxPercentage, removeValueAddedTaxPercentage, currency,
             x, y, elevation, angle, roll, pitch, horizontalAxis, basePlanItem,
             width, depth, height, proportional, modelTransformations,
             this.wallThickness, this.wallDistance, this.wallWidth, this.wallLeft, this.wallHeight, this.wallTop, this.sashes,
@@ -1672,6 +1798,7 @@ public class HomeFurnitureController implements Controller {
     private final String                      name;
     private final Boolean                     nameVisible;
     private final String                      description;
+    private final Map<String, Object>         additionalProperties;
     private final BigDecimal                  price;
     private final boolean                     removePrice;
     private final String                      currency;
@@ -1715,7 +1842,7 @@ public class HomeFurnitureController implements Controller {
                                               Selectable [] oldSelection,
                                               Selectable [] newSelection,
                                               ModifiedPieceOfFurniture [] modifiedFurniture,
-                                              String name, Boolean nameVisible, String description,
+                                              String name, Boolean nameVisible, String description, Map<String, Object> additionalProperties,
                                               BigDecimal price, boolean removePrice, BigDecimal valueAddedTaxPercentage, boolean removeValueAddedTaxPercenage, String currency,
                                               Float x, Float y, Float elevation,
                                               Float angle, Float roll, Float pitch, FurnitureHorizontalAxis horizontalAxis, Boolean basePlanItem,
@@ -1735,6 +1862,7 @@ public class HomeFurnitureController implements Controller {
       this.name = name;
       this.nameVisible = nameVisible;
       this.description = description;
+      this.additionalProperties = additionalProperties;
       this.price = price;
       this.removePrice = removePrice;
       this.valueAddedTaxPercentage = valueAddedTaxPercentage;
@@ -1792,7 +1920,7 @@ public class HomeFurnitureController implements Controller {
     public void redo() throws CannotRedoException {
       super.redo();
       doModifyFurniture(this.modifiedFurniture,
-          this.name, this.nameVisible, this.description,
+          this.name, this.nameVisible, this.description, this.additionalProperties,
           this.price, this.removePrice, this.valueAddedTaxPercentage, this.removeValueAddedTaxPercentage, this.currency,
           this.x, this.y, this.elevation, this.angle, this.roll, this.pitch, this.horizontalAxis, this.basePlanItem,
           this.width, this.depth, this.height, this.proportional, this.modelTransformations,
@@ -1815,7 +1943,7 @@ public class HomeFurnitureController implements Controller {
    * Modifies furniture properties with the values in parameter.
    */
   private static void doModifyFurniture(ModifiedPieceOfFurniture [] modifiedFurniture,
-                                        String name, Boolean nameVisible, String description,
+                                        String name, Boolean nameVisible, String description, Map<String, Object> additionalProperties,
                                         BigDecimal price, boolean removePrice, BigDecimal valueAddedTaxPercentage, boolean removeValueAddedTaxPercenage, String currency,
                                         Float x, Float y, Float elevation,
                                         Float angle, Float roll, Float pitch, FurnitureHorizontalAxis horizontalAxis, Boolean basePlanItem,
@@ -1833,6 +1961,11 @@ public class HomeFurnitureController implements Controller {
       if (nameVisible != null) {
         piece.setNameVisible(nameVisible);
       }
+      if (additionalProperties != null) {
+        for (Map.Entry<String, Object> property : additionalProperties.entrySet()) {
+          piece.setProperty(property.getKey(), property.getValue());
+        }
+      }
       if (description != null) {
         piece.setDescription(description);
       }
@@ -1998,6 +2131,7 @@ public class HomeFurnitureController implements Controller {
     private final String               name;
     private final boolean              nameVisible;
     private final String               description;
+    private final Map<String, Object>  properties;
     private final BigDecimal           price;
     private final BigDecimal           valueAddedTaxPercentage;
     private final String               currency;
@@ -2027,6 +2161,19 @@ public class HomeFurnitureController implements Controller {
       this.name = piece.getName();
       this.nameVisible = piece.isNameVisible();
       this.description = piece.getDescription();
+      Collection<String> propertyNames = piece.getPropertyNames();
+      if (propertyNames.size() == 0) {
+        this.properties = Collections.emptyMap();
+      } else if (propertyNames.size() == 1) {
+        String name = propertyNames.iterator().next();
+        this.properties = Collections.singletonMap(name,
+            (Object)(piece.isContentProperty(name) ? piece.getContentProperty(name) : piece.getProperty(name)));
+      } else {
+        this.properties = new HashMap<String, Object>();
+        for (String name : propertyNames) {
+          this.properties.put(name, piece.isContentProperty(name) ? piece.getContentProperty(name) : piece.getProperty(name));
+        }
+      }
       this.price = piece.getPrice();
       this.valueAddedTaxPercentage = piece.getValueAddedTaxPercentage();
       this.currency = piece.getCurrency();
@@ -2060,6 +2207,16 @@ public class HomeFurnitureController implements Controller {
       this.piece.setName(this.name);
       this.piece.setNameVisible(this.nameVisible);
       this.piece.setDescription(this.description);
+      Collection<String> propertyNames = this.piece.getPropertyNames();
+      for (String name : propertyNames) {
+        this.piece.setProperty(name, this.properties.get(name));
+      }
+      // Reset removed properties
+      for (Map.Entry<String, Object> entry : this.properties.entrySet()) {
+        if (!propertyNames.contains(entry.getKey())) {
+          this.piece.setProperty(entry.getKey(), entry.getValue());
+        }
+      }
       if (!(this.piece instanceof HomeFurnitureGroup)) {
         this.piece.setPrice(this.price);
         this.piece.setValueAddedTaxPercentage(this.valueAddedTaxPercentage);
