@@ -25,6 +25,7 @@ import java.util.Arrays;
 
 import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.HomeMaterial;
+import com.eteks.sweethome3d.model.PieceOfFurniture;
 import com.eteks.sweethome3d.model.Transformation;
 import com.eteks.sweethome3d.model.UserPreferences;
 
@@ -52,7 +53,7 @@ public class ModelMaterialsController implements Controller {
   private float                       modelHeight;
   private float [][]                  modelRotation;
   private Transformation []           modelTransformations;
-  private boolean                     backFaceShown;
+  private int                         modelFlags;
   private HomeMaterial []             materials;
 
   public ModelMaterialsController(String title,
@@ -184,16 +185,34 @@ public class ModelMaterialsController implements Controller {
 
   /**
    * Sets whether the 3D model used to preview materials change should show back face.
+   * @deprecated Prefer use {@link #setModelFlags} with {@link PieceOfFurniture#SHOW_BACK_FACE} flag.
    */
   void setBackFaceShown(boolean backFaceShown) {
-    this.backFaceShown = backFaceShown;
+    setModelFlags((getModelFlags() & ~PieceOfFurniture.SHOW_BACK_FACE)
+        | (backFaceShown ? PieceOfFurniture.SHOW_BACK_FACE : 0));
   }
 
   /**
    * Returns <code>true</code> if the 3D model used to preview materials change should show back face.
    */
   public boolean isBackFaceShown() {
-    return this.backFaceShown;
+    return (this.modelFlags & PieceOfFurniture.SHOW_BACK_FACE) == PieceOfFurniture.SHOW_BACK_FACE;
+  }
+
+  /**
+   * Sets the flags applied to the 3D model used to preview materials change.
+   * @since 7.0
+   */
+  public void setModelFlags(int modelFlags) {
+    this.modelFlags = modelFlags;
+  }
+
+  /**
+   * Returns the flags applied to the 3D model used to preview materials change.
+   * @since 7.0
+   */
+  public int getModelFlags() {
+    return this.modelFlags;
   }
 
   /**
