@@ -219,10 +219,11 @@ public class BackgroundImageWizardTest extends ComponentTestFixture {
   private void runAction(final HomeController controller, 
                          final HomePane.ActionType actionType,
                          JComponentTester tester) {
-    tester.invokeAndWait(new Runnable() {
-      public void run() {
-        ((JComponent)controller.getView()).getActionMap().get(actionType).actionPerformed(null);
-      }
-    });
+    tester.invokeLater(new Runnable() {
+        public void run() {
+          ((JComponent)controller.getView()).getActionMap().get(actionType).actionPerformed(null);
+        }
+      });
+    tester.waitForIdle();
   }
 }
