@@ -21,6 +21,8 @@ package com.eteks.sweethome3d.junit;
 
 import java.awt.Component;
 import java.awt.Dialog;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
 import java.io.File;
 import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
@@ -167,7 +169,7 @@ public class PhotoCreationTest extends ComponentTestFixture {
     // Retrieve PhotoPanel components
     PhotoPanel panel = (PhotoPanel)TestUtilities.findComponent(photoCreationDialog, PhotoPanel.class);    
     JButton createButton = (JButton)TestUtilities.getField(panel, "createButton");
-    JButton saveButton = (JButton)TestUtilities.getField(panel, "saveButton");
+    final JButton saveButton = (JButton)TestUtilities.getField(panel, "saveButton");
     JButton closeButton = (JButton)TestUtilities.getField(panel, "closeButton");
     PhotoSizeAndQualityPanel sizePanel = (PhotoSizeAndQualityPanel)TestUtilities.getField(panel, "sizeAndQualityPanel");
     final JSpinner widthSpinner = (JSpinner)TestUtilities.getField(sizePanel, "widthSpinner");
@@ -183,11 +185,15 @@ public class PhotoCreationTest extends ComponentTestFixture {
       });
     assertEquals("Height spinner has wrong value", 200, ((Number)heightSpinner.getValue()).intValue());
     JSliderTester sliderTester = new JSliderTester();
+    createButton.addActionListener(new ActionListener() {
+        public void actionPerformed(ActionEvent ev) {
+          assertFalse("Rendering didn't start", saveButton.isEnabled());
+        }
+      });
     for (int i = qualitySlider.getMinimum(); i <= qualitySlider.getMaximum(); i++) {
       sliderTester.actionSlide(qualitySlider, i);
       // Test image creation at each quality 
       tester.actionClick(createButton);
-      assertFalse("Rendering didn't start", saveButton.isEnabled());
       // Wait image is generated
       for (int j = 0; j < 1000 && !saveButton.isEnabled(); j++) {
         Thread.sleep(100);
