@@ -19,6 +19,7 @@
  */
 package com.eteks.sweethome3d.junit;
 
+import java.text.DecimalFormatSymbols;
 import java.text.ParseException;
 import java.text.ParsePosition;
 import java.util.Locale;
@@ -31,9 +32,6 @@ import javax.swing.JSpinner;
 import javax.swing.JSpinner.DefaultEditor;
 import javax.swing.undo.UndoableEditSupport;
 
-import junit.framework.ComparisonFailure;
-import junit.framework.TestCase;
-
 import com.eteks.sweethome3d.io.DefaultUserPreferences;
 import com.eteks.sweethome3d.io.FileUserPreferences;
 import com.eteks.sweethome3d.j3d.PhotoRenderer;
@@ -50,6 +48,7 @@ import com.eteks.sweethome3d.tools.OperatingSystem;
 import com.eteks.sweethome3d.tools.URLContent;
 import com.eteks.sweethome3d.viewcontroller.BackgroundImageWizardController;
 import com.eteks.sweethome3d.viewcontroller.CompassController;
+import com.eteks.sweethome3d.viewcontroller.DimensionLineController;
 import com.eteks.sweethome3d.viewcontroller.FurnitureCatalogController;
 import com.eteks.sweethome3d.viewcontroller.FurnitureController;
 import com.eteks.sweethome3d.viewcontroller.HelpController;
@@ -75,6 +74,9 @@ import com.eteks.sweethome3d.viewcontroller.UserPreferencesController;
 import com.eteks.sweethome3d.viewcontroller.VideoController;
 import com.eteks.sweethome3d.viewcontroller.WallController;
 
+import junit.framework.ComparisonFailure;
+import junit.framework.TestCase;
+
 /**
  * Tests {@link com.eteks.sweethome3d.swing.UserPreferencesPanel user preferences panel}.
  * @author Emmanuel Puybaret
@@ -156,7 +158,8 @@ public class UserPreferencesPanelTest extends TestCase {
     catalogIconRadioButton.setSelected(true);
     monochromeRadioButton.setSelected(true);
     ((DefaultEditor)newWallThicknessSpinner.getEditor()).getTextField().setText("1\u215C\"");
-    ((DefaultEditor)newHomeWallHeightSpinner.getEditor()).getTextField().setText("8'4,25");
+    char decimalSeparator = new DecimalFormatSymbols(new Locale(preferences.getLanguage())).getDecimalSeparator();
+    ((DefaultEditor)newHomeWallHeightSpinner.getEditor()).getTextField().setText("8'4" + decimalSeparator + "25");
 
     // 4. Retrieve panel values into preferences
     controller.modifyUserPreferences();
@@ -406,6 +409,7 @@ public class UserPreferencesPanelTest extends TestCase {
       new WallController(home, preferences, viewFactory, contentManager, undoableEditSupport).getView();
       new RoomController(home, preferences, viewFactory, contentManager, undoableEditSupport).getView();
       new PolylineController(home, preferences, viewFactory, contentManager, undoableEditSupport).getView();
+      new DimensionLineController(home, preferences, viewFactory, undoableEditSupport).getView();
       new LabelController(home, preferences, viewFactory, undoableEditSupport).getView();
       new CompassController(home, preferences, viewFactory, undoableEditSupport).getView();
       new ObserverCameraController(home, preferences, viewFactory).getView();
